#ifndef UNIX
#define DEC_OS
#endif

/*
 *   getsym symbol descriptors.
 */

#define	NUL	0		/* null	descriptor */
#define	LBL	1		/* label */
#define	CON	2		/* constant */
#define	SPA	3		/* spaces */
#define	DEL	4		/* delimiter */
#define	EOL	5		/* end of line */
#define	EORN	6		/* end of file */
#define	COM	7		/* comment */
#define	OPR	8		/* operator */
#define	STR	9		/* string */
#define	MCH	10		/* global match	*/

#define	EXP	20		/* for mactab only - will NEVER	appear */
#define	SKP	21		/* no match symbol */

#define	LIT	22		/* literal */
#define	CHR	23		/* extra character */


/*
 *   pass1 action table	descriptors
 */
#define	NOOP	0		/* no-op */
#define	ALBL	1		/* add label to	ifield */
#define	DOPV	2		/* decode op & value */
#define	OLBL	3		/* out label to	i_opr */
#define	OOPR	4		/* out opr */
#define	ODEL	5		/* out del ',' */
#define	OCON	6		/* out constant	*/
#define	OCHR	7		/* out character */
#define	OSTR	8		/* output string */
#define	ENDP	9		/* end pass */
#define	SELC	10		/* select options */
#define	EXPR	11		/* call	expr parser */
#define	RETN	12		/* return from call */
#define	GOTO	13		/* fail	return */
#define	OERR	14		/* error on scan */
#define	OREC	15		/* out i_rec (ok) */

/*
 *   symbol table mode descriptors.
 *
 *   IDXMSK are the bits in the mode descriptor used to store
 *   the index of the global/external symbol for the external
 *   symbol table.
 *
 *	REFR	= sym referenced in operand
 *	DEFN 	= symbol defined a label, or EXPORT/EXTERN psueudo ops
 *	EXPO	= defined by EXPORT pseudo-op
 *	EXT	= declared in EXTERN pseudo-op or label with 2 colons (::)
 *	LITR	= symbol is a constant
 *	REL	= 1 = PC rel  0 = Absolute
 *
 *	IDXMSK	= the number of bits availible for global symbol index val
 */

#define	REFR	0x8000
#define	DEFN	0x4000
#define	EXPO	0x2000
#define	EXT	0x1000
#define	LITR	0x0800
#define	REL	0x0400
#define	ABS	0x0200

#define IDXMSK	0x03ff

/*
 *
 */
#define	PMODE	0x1000
#define	RMODE	0x2000

/*
 *   misc. descriptors.
 */

#define	MWORD	(011224)	/* m.out magic number for 8 bit output */
				/* modified Oct 85 aek */

#define	WORDSIZ	16		/* word	size of	host in	bits */

#define	RMAGIC	033221		/* rfile magic number */

#define	ERR	-1		/* error or undef */
#define	MAXBUF	256		/* general usage buffer	size */
#define	MAXOPN	32		/* longest single operand */
#define	LCOUNT	8		/* number of locn counters */
#define	TRUE	1		/* some	truth value */
#define	FALSE	0		/* zero	=> false */
#define	SELFMT	0x01		/* select format */
#define	SELOPC	0x02		/* select opcode class */

#define	OP	0x0001		/* opcode bit */
#define	PS	0x0002		/* pseudo bit */

#define	SNAMLEN 6		/* length of symbol name */

/*
 *   struct definition
 */
struct	st	{			/* symbol table	*/
	char	s_name[SNAMLEN];	/* symbol name */
	int	s_value;		/* actual value	*/
	int	s_mode;			/* symbol usage	*/
	struct	st	*s_loc;		/* local symbol blk */
	struct	st	*s_next;	/* next	in chain */
	};

struct	it	{			/* intermediate	code */
	int	i_flags;		/* tag bits - op, pseudo */
	int	i_label;		/* label index to symtab */
	int	i_op;			/* opcode table	entry */
	int	i_loc;			/* current loc cntr # */
	char	i_selc[4];		/* action selection */
	char	i_opr[MAXOPN];		/* operand description */
	};

struct	lt	{			/* location counter table */
	int	l_value;		/* lcntr value */
	int	l_attrib;		/* segment attributes */
	int	l_start;		/* start addr of code */
	int	l_limit;		/* limit at eo pass1 */
	char	*l_next;		/* next	object word */
	int	*l_rel_f;		/* relocation factor */
	int	*l_reloc;		/* relocation info */
	char	*l_rel_n;		/* pointer to rel info */
	};

struct	fd	{			/* format descriptor */
	char	f_class;		/* instr class */
	char	f_len;			/* len of instr	in bu's	*/
	int	f_desc[8];		/* descriptor type */
	int	f_value[8];		/* values */
	int	f_width[8];		/* widths of bits */
	};

struct	bt	{			/* i/o buffer */
	int	b_fdesc;		/* file	desc */
	int	b_left;			/* nleft */
	char	*b_next;		/* next	*/
	char	b_buf[512];		/* char	buffer */
	};

struct	ht	{			/* header record */
	int	h_magic;		/* rfile magic # */
	int	h_literals;		/* # of	literals */
	int	h_labels;		/* # pre-def labs */
	int	h_formats;		/* # of	f desc */
	int	h_ops;			/* # of	ops */
	struct	fd	dctype[5];
	int	h_pc_post;		/* pc incr */
	int	h_bu_len;		/* len of bu in	bits */
	int	h_w_len;		/* len of wrd in bu */
	int	h_o_len;		/* opcode struct len */
	int	h_p_len;		/* len of parser table */
	int	h_p_start;		/* start of parser */
	int	h_page;			/* page	len */
	int	h_ii;			/* selected ill. instr op */
	char	h_mac[MAXOPN];		/* title */
	};

struct	od	{			/* opcode descriptors */
	char	o_name[8];		/* opcode symbolic */
	struct	{
		int	o_value;	/* real	value */
		int	o_format;	/* format to use */
		}	o_code[1];
	};

struct	tbl	{			/* parser table	*/
	int	tb_sym;			/* symbol type */
	int	tb_mem;			/* member type */
	int	tb_next;		/* next	entry */
	int	tb_act;			/* action before next case */
	char	tb_arg[4];		/* args	for action */
	};

/*
 * table offsets for the pseudo ops
 */

#define	DC1	0
#define	DC2	1
#define	DC3	2
#define	DC4	3
#define	EQU	4
#define	EXPORT	5
#define	EXTERN	6
#define	ALIGN	7
#define	ORG	8
#define	ASECT	9
#define	PSECT	10
#define	DS	11
#define	END	12
#define	TITLE	13
#define	EJECT	14
#define	DCS	15
#define	STRUCT	16
#define	ENDS	17
#define	IFDEF	18
#define	ENDIF	19
#define	LIST	20
#define	NLIST	21


#define OD	(sizeof (struct od))
#define	ST	(sizeof	(struct	st))
#define	IT	(sizeof	(struct	it))
#define	LT	(sizeof	(struct	lt))
#define	FD	(sizeof	(struct	fd))
#define	BT	(sizeof	(struct	bt))
#define	HT	(sizeof	(struct	ht))
#define	TBL	(sizeof	(struct	tbl))
#define	INT	(sizeof	(int))

#define	BITMASK(m)	(bitmask[(m)])

#define	MAXFILES	30		/* max number of src files */

/*
 * m.out structure
 *
 * file consists of outhdr followed by ao_segcnt segment descriptors
 * followed by any non-zero code segments, optionally relocation info
 * and the global symbol table entries
 *
 */

struct	out{
	int	ou_length;		/* segment length */
	int	ou_start;		/* starting address */
	int	ou_attrib;		/* segment attributes */
	};

#define	HASREL	0x100			/* obj file has relocation bits */
#define HASSYM	0x200			/* obj file has symbol table    */

struct	outhdr{
	int	ao_mword;		/* magic number */
	int	ao_bu_len;		/* byte width */
	int	ao_attrib;		/* segment count */
	};

#define AOUT	(sizeof (struct outhdr))
#define OUT	(sizeof (struct out))

